#include "Texture.h"
#include <IL/il.h>
#include "glInterface.h"
#include <assert.h>
#include <iostream>


namespace CHCDemo
{

using namespace std;


void startil()
{
	ilInit();
	assert(ilGetError() == IL_NO_ERROR);
}


void stopil()
{
	ilShutDown();
	assert(ilGetError() == IL_NO_ERROR);
}


Texture::Texture(const std::string &filename):
mWidth(0), mHeight(0), mFormat(0), mImage(NULL), mTexId(-1), mName(filename)
{
	startil();
	
	if (!ilLoadImage(ILstring(filename.c_str())))
	{
		cerr << "Image load error " << ilGetError() << ": " << filename << endl;
	}
	else
	{
		mWidth = ilGetInteger(IL_IMAGE_WIDTH);
		mHeight = ilGetInteger(IL_IMAGE_HEIGHT);

		mFormat = FORMAT_RGBA;
		mImage = malloc(GetByteSize());

		cout << "successfully loaded texture " << filename << endl;
		ilCopyPixels(0, 0, 0, mWidth, mHeight, 1, IL_RGBA, IL_UNSIGNED_BYTE, mImage);
		assert(ilGetError() == IL_NO_ERROR);
	}
	
	stopil();

	Create();
}


void Texture::Create()
{
	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &mTexId);
	glBindTexture(GL_TEXTURE_2D, mTexId);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, mWidth, mHeight,
					  GL_RGBA, GL_UNSIGNED_BYTE, mImage);

	//glTexImage2D(GL_TEXTURE_2D, 0, 3, mWidth, mHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, mImage);
}


int Texture::GetFormat() const
{
	return mFormat;
}


int Texture::GetWidth() const
{
	return mWidth;
}


int Texture::GetHeight() const
{
	return mHeight;
}


void *Texture::GetImage() const
{
	return mImage;
}


Texture::~Texture()
{
	free(mImage);
}


std::string Texture::GetName() const
{
	return mName;
}


int Texture::GetRowLength() const
{
	if (mFormat == FORMAT_RGBA)
		return mWidth * 4 * sizeof (unsigned char);
	else
		return mWidth * 3 * sizeof (unsigned char);
}


int Texture::GetPixelSize() const
{
	if (mFormat == FORMAT_RGBA)
		return 4 * sizeof(unsigned char);
	else 
		return 3 * sizeof(unsigned char);
}


int Texture::GetByteSize() const
{
	return mHeight * GetRowLength();
}


void Texture::Bind() const
{
	glBindTexture(GL_TEXTURE_2D, mTexId);
}

}